from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="CreatePostEphemeralJsonBodyPost")


@attr.s(auto_attribs=True)
class CreatePostEphemeralJsonBodyPost:
    """Post object to create

    Attributes:
        channel_id (str): The channel ID to post in
        message (str): The message contents, can be formatted with Markdown
    """

    channel_id: str
    message: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channel_id = self.channel_id
        message = self.message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "channel_id": channel_id,
                "message": message,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        channel_id = d.pop("channel_id")

        message = d.pop("message")

        create_post_ephemeral_json_body_post = cls(
            channel_id=channel_id,
            message=message,
        )

        create_post_ephemeral_json_body_post.additional_properties = d
        return create_post_ephemeral_json_body_post

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
