from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="CreateTeamJsonBody")


@attr.s(auto_attribs=True)
class CreateTeamJsonBody:
    """
    Attributes:
        name (str): Unique handler for a team, will be present in the team URL
        display_name (str): Non-unique UI name for the team
        type (str): `'O'` for open, `'I'` for invite only
    """

    name: str
    display_name: str
    type: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        display_name = self.display_name
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "display_name": display_name,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        display_name = d.pop("display_name")

        type = d.pop("type")

        create_team_json_body = cls(
            name=name,
            display_name=display_name,
            type=type,
        )

        create_team_json_body.additional_properties = d
        return create_team_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
