from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="InstallMarketplacePluginJsonBody")


@attr.s(auto_attribs=True)
class InstallMarketplacePluginJsonBody:
    """
    Attributes:
        id (str): The ID of the plugin to install.
        version (str): The version of the plugin to install.
    """

    id: str
    version: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "version": version,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        version = d.pop("version")

        install_marketplace_plugin_json_body = cls(
            id=id,
            version=version,
        )

        install_marketplace_plugin_json_body.additional_properties = d
        return install_marketplace_plugin_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
