from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="OwnerInfo")


@attr.s(auto_attribs=True)
class OwnerInfo:
    """
    Attributes:
        user_id (str): A unique, 26 characters long, alphanumeric identifier for the owner. Example:
            ahz0s61gh275i7z2ag4g1ntvjm.
        username (str): Owner's username. Example: aaron.medina.
    """

    user_id: str
    username: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        username = self.username

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "username": username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("user_id")

        username = d.pop("username")

        owner_info = cls(
            user_id=user_id,
            username=username,
        )

        owner_info.additional_properties = d
        return owner_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
